package com.loit.getway.swagger.config;

import com.loit.getway.config.GateWayConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.config.GatewayProperties;
import org.springframework.cloud.gateway.route.RouteLocator;
import org.springframework.cloud.gateway.support.NameUtils;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;
import springfox.documentation.swagger.web.SwaggerResource;
import springfox.documentation.swagger.web.SwaggerResourcesProvider;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: TODO(GateWay 获取SwaggerWay Api-doc)
 * @author yangwenbin
 * @date  2020-04-02
 */

@Component
@Primary
public class SwaggerProvider implements SwaggerResourcesProvider {
  public static final String API_URI = "/v2/api-docs";
  private  RouteLocator routeLocator;
  private  GatewayProperties gatewayProperties;
  @Autowired
  private GateWayConfig gateWayConfig;


  public SwaggerProvider(RouteLocator routeLocator, GatewayProperties gatewayProperties){
    this.gatewayProperties = gatewayProperties;
    this.routeLocator =routeLocator;
  }
  @Override
  public List<SwaggerResource> get() {
    List<SwaggerResource> resources = new ArrayList<>();
    List<String> routes = new ArrayList<>();
    if(gateWayConfig.getSwaggerClose() == null || !gateWayConfig.getSwaggerClose()){
      routeLocator.getRoutes().subscribe(route -> routes.add(route.getId()));
      gatewayProperties.getRoutes().stream().filter(routeDefinition -> routes.contains(routeDefinition.getId()))
              .forEach(routeDefinition -> routeDefinition.getPredicates().stream()
                      .filter(predicateDefinition -> ("Path").equalsIgnoreCase(predicateDefinition.getName()))
                      .forEach(predicateDefinition -> resources.add(swaggerResource(routeDefinition.getId(),
                              predicateDefinition.getArgs().get(NameUtils.GENERATED_NAME_PREFIX + "0")
                                      .replace("/**", API_URI)))));
    }

    return resources;
  }

  private SwaggerResource swaggerResource(String name, String location) {
    SwaggerResource swaggerResource = new SwaggerResource();
    swaggerResource.setName(name);
    swaggerResource.setLocation(location);
    swaggerResource.setSwaggerVersion("2.0");
    return swaggerResource;
  }
}
